@@ -6,15 +6,17 @@
 (*            Pierre Weis, projet Cristal, INRIA Rocquencourt          *)
 (*            Jun Furuse, projet Cristal, INRIA Rocquencourt           *)
 (*                                                                     *)
-(*  Copyright 1999,2000,2001,2002,2001,2002                            *)
+(*  Copyright 1999-2004                                                *)
 (*  Institut National de Recherche en Informatique et en Automatique.  *)
 (*  Distributed only by permission.                                    *)
 (*                                                                     *)
 (***********************************************************************)
 
+(* $Id: bmp.ml,v 1.12 2004/09/07 07:31:01 weis Exp $ *)
+
 (* Loading and saving image in the bmp format. *)
 
-open Image;;
+open Images;;
 
 (*
   Caml representation of a bmp bit map image.
@@ -29,29 +31,29 @@ open Image;;
 type bmp = {
    bmpFileHeader : bitmapfileheader;           (* Bytes <0  14< *)
    bmpInfoHeader : bitmapinfoheader;           (* Bytes <14 54< *)
-   bmpRgbQuad : rgb array;               (* Bytes <54 ... *)
+   bmpRgbQuad : rgb array;                     (* Bytes <54 ... *)
    bmpBytes : string;                          (* Bytes <bfOffBits ... *)
 }
 
 and bitmapfileheader = {
-    (* WORD: that is 2 bytes *) bfType : int ; (* Bytes <0   2< *)
-    (* DWORD: that is 2 WORDs *) bfSize : int ;(* Bytes <2   6< *)
-    (* WORD *) bfReserved1 : int ;             (* Bytes <6   8< *)
-    (* WORD *) bfReserved2 : int ;             (* Bytes <8  10< *)
-    (* DWORD *) bfOffBits : int                (* Bytes <10 14< *)
+    (* WORD: that is 2 bytes *) bfType : int;  (* Bytes <0   2< *)
+    (* DWORD: that is 2 WORDs *) bfSize : int; (* Bytes <2   6< *)
+    (* WORD *) bfReserved1 : int;              (* Bytes <6   8< *)
+    (* WORD *) bfReserved2 : int;              (* Bytes <8  10< *)
+    (* DWORD *) bfOffBits : int;               (* Bytes <10 14< *)
 }
 
 and bitmapinfoheader = {
-    (* DWORD *) biSize : int ;                 (* Bytes <14 18< *)
-    (* DWORD *) biWidth : int ;                (* Bytes <18 22< *)
-    (* DWORD *) biHeight : int ;               (* Bytes <22 26< *)
-    (* WORD *) biPlanes  : int ;               (* Bytes <26 28< *)
-    (* WORD *) biBitCount : bibitcount ;       (* Bytes <28 30< *)
-    (* DWORD *) biCompression : bicompression ;(* Bytes <30 34< *)
-    (* DWORD *) biSizeImage : int ;            (* Bytes <34 38< *)
-    (* DWORD *) biXPelsPerMeter : int ;        (* Bytes <38 42< *)
-    (* DWORD *) biYPelsPerMeter : int ;        (* Bytes <42 46< *)
-    (* DWORD *) biClrUsed : int ;              (* Bytes <46 50< *)
+    (* DWORD *) biSize : int;                  (* Bytes <14 18< *)
+    (* DWORD *) biWidth : int;                 (* Bytes <18 22< *)
+    (* DWORD *) biHeight : int;                (* Bytes <22 26< *)
+    (* WORD *) biPlanes  : int;                (* Bytes <26 28< *)
+    (* WORD *) biBitCount : bibitcount;        (* Bytes <28 30< *)
+    (* DWORD *) biCompression : bicompression; (* Bytes <30 34< *)
+    (* DWORD *) biSizeImage : int;             (* Bytes <34 38< *)
+    (* DWORD *) biXPelsPerMeter : int;         (* Bytes <38 42< *)
+    (* DWORD *) biYPelsPerMeter : int;         (* Bytes <42 46< *)
+    (* DWORD *) biClrUsed : int;               (* Bytes <46 50< *)
     (* DWORD *) biClrImportant : int           (* Bytes <50 54< *)
 }
 
@@ -101,17 +103,16 @@ and bibitcount =
 
 
 (* # of bytes read during loading : should be equal to the size of the file *)
-let bytes_read = ref 0 ;;
+let bytes_read = ref 0;;
 
-let read_byte ic = incr bytes_read ; input_byte ic ;;
+let read_byte ic = incr bytes_read; input_byte ic;;
 let skip_byte ic = incr bytes_read; let _ = input_byte ic in ();;
 
 let read_word ic =
  let b0 = read_byte ic in
  let b1 = read_byte ic in
  (* little-endian form *)
- (b1 lsl 8) + b0
-;;
+ (b1 lsl 8) + b0;;
 
 let read_dword ic =
  let b0 = read_byte ic in
@@ -119,8 +120,7 @@ let read_dword ic =
  let b2 = read_byte ic in
  let b3 = read_byte ic in
  (* little-endian form *)
- (b3 lsl 24) + (b2 lsl 16) + (b1 lsl 8) + b0
-;;
+ (b3 lsl 24) + (b2 lsl 16) + (b1 lsl 8) + b0;;
 
 let read_bit_count ic =
  match read_word ic with
@@ -159,19 +159,18 @@ let load_rgbquad ic =
  let g = read_byte ic in
  let r = read_byte ic in
  let u = read_byte ic in
- { b = b ; g = g ; r = r }
-;;
+ { b = b; g = g; r = r };;
 
 let load_bitmapfileheader ic =
  let bfType = read_word ic in
- if bfType <> 19778 (* BM *) then failwith "Invalid file tag" ;
+ if bfType <> 19778 (* BM *) then failwith "Invalid file tag";
  let bfSize = read_dword ic in
  let bfReserved1 = read_word ic in
  let bfReserved2 = read_word ic in
  let bfOffBits = read_dword ic in
- { bfType = bfType ; bfSize = bfSize ; bfReserved1 = bfReserved1 ;
-   bfReserved2 = bfReserved2 ; bfOffBits = bfOffBits }
-;;
+ { bfType = bfType; bfSize = bfSize; bfReserved1 = bfReserved1;
+   bfReserved2 = bfReserved2; bfOffBits = bfOffBits;
+ };;
 
 let load_bitmapinfoheader ic =
  (* Found a tagBITMAPINFO *)
@@ -187,12 +186,12 @@ let load_bitmapinfoheader ic =
  let biClrUsed = read_dword ic in
  let biClrImportant = read_dword ic in
  (* header = tagBITMAPINFOHEADER *)
- { biSize = biSize ; biWidth = biWidth ; biHeight = biHeight ;
-   biPlanes = biPlanes ; biBitCount = biBitCount ;
-   biCompression = biCompression ; biSizeImage = biSizeImage ;
-   biXPelsPerMeter = biXPelsPerMeter ; biYPelsPerMeter = biYPelsPerMeter ;
-   biClrUsed = biClrUsed ; biClrImportant = biClrImportant }
-;;
+ { biSize = biSize; biWidth = biWidth; biHeight = biHeight;
+   biPlanes = biPlanes; biBitCount = biBitCount;
+   biCompression = biCompression; biSizeImage = biSizeImage;
+   biXPelsPerMeter = biXPelsPerMeter; biYPelsPerMeter = biYPelsPerMeter;
+   biClrUsed = biClrUsed; biClrImportant = biClrImportant;
+ };;
 
 let load_colors bfh bih ic =
  (* Reading RGBQUADs *)
@@ -224,10 +223,10 @@ let load_image8data bih ic =
        let bitmapindex = ref (i * bih.biWidth) in
        for j = 0 to pad - 1 do
          let c = Char.chr (read_byte ic) in
-         if j < bih.biWidth then bitmap.[!bitmapindex] <- c ;
+         if j < bih.biWidth then bitmap.[!bitmapindex] <- c;
          incr bitmapindex
          done
-       done ;
+       done;
      bitmap
  | BI_RLE8 ->
      (* Run-length encoded format for bitmaps with 8 bits per pixel *)
@@ -245,8 +244,8 @@ let load_image8data bih ic =
            match read_byte ic with
            | 0 ->
                (* End of line code *)
-               x := 0 ;
-               incr y ;
+               x := 0;
+               incr y;
                bitmapindex := !x + (bih.biHeight - !y - 1) * bih.biWidth
            | 1 ->
                (* End of bitmap : force exit *)
@@ -254,9 +253,9 @@ let load_image8data bih ic =
            | 2 ->
                (* Delta *)
                let c1 = read_byte ic in
-               x := !x + c1 ;
+               x := !x + c1;
                let c2 = read_byte ic in
-               y := !y + c2 ;
+               y := !y + c2;
                bitmapindex := !x + (bih.biHeight - !y - 1) * bih.biWidth
            | c ->
                (* c should be between 03H and FFH *)
@@ -265,10 +264,10 @@ let load_image8data bih ic =
                   each of which contains the color index of a single pixel. *)
                for i = 0 to c - 1 do
                  let c1 = read_byte ic in
-                 bitmap.[!bitmapindex] <- Char.chr c1 ;
-                 incr x ;
+                 bitmap.[!bitmapindex] <- Char.chr c1;
+                 incr x;
                  incr bitmapindex
-               done ;
+               done;
                (* Odd length run: read an extra pad byte *)
                if c land 1 <> 0 then skip_byte ic
            end
@@ -276,11 +275,11 @@ let load_image8data bih ic =
            (* Encoded mode *)
            let c1 = read_byte ic in
            for i = 0 to c - 1 do
-             bitmap.[!bitmapindex] <- Char.chr c1 ;
-             incr x ;
+             bitmap.[!bitmapindex] <- Char.chr c1;
+             incr x;
              incr bitmapindex
            done
-     done ;
+     done;
      bitmap
  | BI_RLE4 ->
      failwith ("Invalid compression mode : BI_RLE4")
@@ -298,18 +297,18 @@ let load_image1data bih ic =
    for j = 0 to pad - 1 do
      if !bnum land 7 = 0 then
       begin
-       c := read_byte ic ;
-       bnum := 0 ;
-      end ;
+       c := read_byte ic;
+       bnum := 0;
+      end;
      if j < bih.biWidth then
       begin
-       bitmap.[!bitmapindex] <- if !c land 0x80 <> 0 then '\001' else '\000' ;
-       incr bitmapindex ;
+       bitmap.[!bitmapindex] <- if !c land 0x80 <> 0 then '\001' else '\000';
+       incr bitmapindex;
        c := !c lsl 1;
-      end ;
+      end;
      incr bnum
     done
- done ;
+ done;
  bitmap
 ;;
 
@@ -328,18 +327,18 @@ let load_image4data bih ic =
         if !nyblenum land 1 = 0 then
           begin
           (* Read the next byte *)
-          c := read_byte ic ;
+          c := read_byte ic;
           nyblenum := 0
-          end ;
+          end;
         if j < bih.biWidth then
           begin
-          bitmap.[!bitmapindex] <- Char.chr ((!c land 0xf0) lsr 4) ;
-          incr bitmapindex ;
+          bitmap.[!bitmapindex] <- Char.chr ((!c land 0xf0) lsr 4);
+          incr bitmapindex;
           c := !c lsl 4
-          end ;
+          end;
         incr nyblenum
       done
-    done ;
+    done;
     bitmap
  | BI_RLE4 ->
     let x = ref 0 in
@@ -355,7 +354,7 @@ let load_image4data bih ic =
           | 0 -> 
               (* End of line *)
               x := 0;
-              incr y ;
+              incr y;
               bitmapindex := !x + (bih.biHeight - !y - 1) * bih.biWidth
           | 1 ->
               (* End of bitmap : force exit *)
@@ -363,19 +362,19 @@ let load_image4data bih ic =
           | 2 ->
               (* Delta *)
               let c' = read_byte ic in
-              x := !x + c' ;
+              x := !x + c';
               let c'' = read_byte ic in
-              y := !y + c'' ;
+              y := !y + c'';
               bitmapindex := !x + (bih.biHeight - !y - 1) * bih.biWidth
           | c ->
               (* Absolute mode *)
               for i = 0 to c - 1 do
-                if i land 1 = 0 then c1 := read_byte ic ;
+                if i land 1 = 0 then c1 := read_byte ic;
                 let c = if i land 1 <> 0 then !c1 else !c1 lsr 4 in
-                bitmap.[!bitmapindex] <- Char.chr (c land 0x0F) ;
-                incr x ;
+                bitmap.[!bitmapindex] <- Char.chr (c land 0x0F);
+                incr x;
                 incr bitmapindex
-              done ;
+              done;
               (* Read pad byte *)
               if c land 3 = 1 || c land 3 = 2 then skip_byte ic
           end
@@ -388,10 +387,10 @@ let load_image4data bih ic =
           for i = 0 to c - 1 do
             let c = if i land 1 <> 0 then col1 else col2 in
             bitmap.[!bitmapindex] <- Char.chr c;
-            incr x ;
+            incr x;
             incr bitmapindex
           done
-    done ;
+    done;
     bitmap
  | BI_RLE8 ->
     failwith ("Invalid compression mode : BI_RLE8")
@@ -403,15 +402,15 @@ let load_image24data bih ic =
   let pad = (4 - ((bih.biWidth * 3) mod 4)) land 0x03 in
   let pp = ref 0 in
   for i = bih.biHeight - 1 downto 0 do
-    pp := (i * bih.biWidth * 3) ;
+    pp := (i * bih.biWidth * 3);
     for j = 0 to bih.biWidth - 1 do
-      bitmap.[!pp + 2] <- Char.chr (read_byte ic) ;   (* Blue *)
-      bitmap.[!pp + 1] <- Char.chr (read_byte ic) ;   (* Green *)
-      bitmap.[!pp] <- Char.chr (read_byte ic) ;     (* Red *)
+      bitmap.[!pp + 2] <- Char.chr (read_byte ic);   (* Blue *)
+      bitmap.[!pp + 1] <- Char.chr (read_byte ic);   (* Green *)
+      bitmap.[!pp] <- Char.chr (read_byte ic);     (* Red *)
       pp := !pp + 3
-    done ;
-    for j = 0 to pad - 1 do skip_byte ic done ;
-  done ;
+    done;
+    for j = 0 to pad - 1 do skip_byte ic done;
+  done;
   bitmap
 ;;
 
@@ -465,9 +464,9 @@ let image_of_bmp = function
    bmpBytes = bitmap } ->
    match bih.biBitCount with
    | ColorM ->
-       Rgb24 (Rgb24.create_with bih.biWidth bih.biHeight bitmap)
+       Rgb24 (Rgb24.create_with bih.biWidth bih.biHeight [] bitmap)
    | _ ->
-       Index8 (Index8.create_with bih.biWidth bih.biHeight 
+       Index8 (Index8.create_with bih.biWidth bih.biHeight []
 		 {map= colormap; max= 256} (-1) bitmap)
 ;;
 
@@ -480,7 +479,7 @@ let load filename opts = image_of_bmp (load_bmp filename);;
 
 let bytes_written = ref 0;;
 
-let write_byte oc b = incr bytes_written; output_byte oc b ;;
+let write_byte oc b = incr bytes_written; output_byte oc b;;
 
 let output_word oc w =
  (* little-endian form *)
@@ -531,10 +530,10 @@ let write_rgbquad oc rgb =
  write_byte oc u;;
 
 let write_bmpFileHeader oc = function {
-    (* WORD *) bfType = bft ;
+    (* WORD *) bfType = bft;
     (* DWORD *) bfSize = bfs;
-    (* WORD *) bfReserved1 = bfr1 ;
-    (* WORD *) bfReserved2 = bfr2 ;
+    (* WORD *) bfReserved1 = bfr1;
+    (* WORD *) bfReserved2 = bfr2;
     (* DWORD *) bfOffBits = bfob
  } ->
    let start_index = !bytes_written in
@@ -549,16 +548,16 @@ let write_bmpFileHeader oc = function {
    start_index, bfSize_index, bfOffBits_index, end_bmpFileHeader;;
 
 let write_bmpInfoHeader oc = function {
-    (* DWORD *) biSize = bis ;
-    (* DWORD *) biWidth = biw ;
-    (* DWORD *) biHeight = bih ;
-    (* WORD *) biPlanes  = bip ;
-    (* WORD *) biBitCount = bibc ;
-    (* DWORD *) biCompression = bic ;
-    (* DWORD *) biSizeImage = bisi ;
-    (* DWORD *) biXPelsPerMeter = bixpm ;
-    (* DWORD *) biYPelsPerMeter = biypm ;
-    (* DWORD *) biClrUsed = bicu ;
+    (* DWORD *) biSize = bis;
+    (* DWORD *) biWidth = biw;
+    (* DWORD *) biHeight = bih;
+    (* WORD *) biPlanes  = bip;
+    (* WORD *) biBitCount = bibc;
+    (* DWORD *) biCompression = bic;
+    (* DWORD *) biSizeImage = bisi;
+    (* DWORD *) biXPelsPerMeter = bixpm;
+    (* DWORD *) biYPelsPerMeter = biypm;
+    (* DWORD *) biClrUsed = bicu;
     (* DWORD *) biClrImportant = bici
  } ->
    let biSize_index = !bytes_written in
@@ -575,8 +574,7 @@ let write_bmpInfoHeader oc = function {
    write_dword oc bicu;
    write_dword oc bici;
    let end_bmpInfoHeader = !bytes_written in
-   biSize_index, biSizeImage_index, end_bmpInfoHeader
-;;
+   biSize_index, biSizeImage_index, end_bmpInfoHeader;;
 
 let write_colors oc color_map =
  (* If color_map is empty, should output a NULL character *)
@@ -648,8 +646,7 @@ let write_image1data bmp oc =
   write_pad oc extra_padding_bytes;
  done;
  let end_bitmap_index = !bytes_written in
- start_bitmap_index, end_bitmap_index
-;;
+ start_bitmap_index, end_bitmap_index;;
 
 let write_image24data bmp oc =
  let bih = bmp.bmpInfoHeader in
@@ -680,8 +677,7 @@ let write_image24data bmp oc =
   write_pad oc extra_padding_bytes;
  done;
  let end_bitmap_index = !bytes_written in
- start_bitmap_index, end_bitmap_index
-;;
+ start_bitmap_index, end_bitmap_index;;
 
 let write_image4data bmp oc =
  let bih = bmp.bmpInfoHeader in
@@ -748,8 +744,7 @@ let write_image4data bmp oc =
     start_bitmap_index, end_bitmap_index
 
  | BI_RLE8 ->
-    failwith ("Invalid compression mode : BI_RLE8")
-;;
+    failwith ("Invalid compression mode : BI_RLE8");;
 
 let write_image8data bmp oc =
  let bih = bmp.bmpInfoHeader in
@@ -811,8 +806,7 @@ let write_image8data bmp oc =
     start_bitmap_index, end_bitmap_index
 
  | BI_RLE4 ->
-    failwith ("Invalid compression mode : BI_RLE8")
-;;
+    failwith ("Invalid compression mode : BI_RLE8");;
 
 let write_image_data oc bmp =
  let bih = bmp.bmpInfoHeader in
@@ -820,45 +814,43 @@ let write_image_data oc bmp =
  | Monochrome -> write_image1data bmp oc
  | Color16 -> write_image4data bmp oc
  | Color256 -> write_image8data bmp oc
- | ColorM -> write_image24data bmp oc
-;;
+ | ColorM -> write_image24data bmp oc;;
 
 let bmp_of_image img =
   match img with
   | Rgb24 bitmap ->  
-    let biW = bitmap.Rgb24.width
-    and biH = bitmap.Rgb24.height
-    and data = Bitmap.dump bitmap.Rgb24.data
-    in
+    let biW = bitmap.Rgb24.width 
+    and biH = bitmap.Rgb24.height 
+    and data = Rgb24.dump bitmap in
     let bfh = {
-      (* WORD *) bfType = 19778 (* BM *) ;
+      (* WORD *) bfType = 19778 (* BM *);
       (* DWORD *) bfSize = -1 (* Unknown to be updated *);
-      (* WORD *) bfReserved1 = 0 ;
-      (* WORD *) bfReserved2 = 0 ;
+      (* WORD *) bfReserved1 = 0;
+      (* WORD *) bfReserved2 = 0;
       (* DWORD *) bfOffBits = -1 (* Unknown to be updated *)
     } in
 
     let bih =
       { (* The size in bytes of this header. *)
-        biSize = -1 ;  (* Unknown to be updated *)
+        biSize = -1;  (* Unknown to be updated *)
         (* Width and height of the image *)
-        biWidth = biW ; biHeight = biH ;
+        biWidth = biW; biHeight = biH;
         (* According to the format, Must be set to 1. *)
-        biPlanes = 1 ;
+        biPlanes = 1;
         (* 24 bits pixels. *)
-        biBitCount = ColorM ;
+        biBitCount = ColorM;
         (* Compression is no compression: we output pixels as
            rgb rgb ... with padding. *)
-        biCompression = BI_RGB ;
+        biCompression = BI_RGB;
         (* The size of the actual image pixels representation in the
            file. Due to padding, cannot be computed here. *)
-        biSizeImage = -1 (* Unknown to be updated *) ;
+        biSizeImage = -1 (* Unknown to be updated *);
         (* This should be OK *)
-        biXPelsPerMeter = 600 ; biYPelsPerMeter = 600 ;
+        biXPelsPerMeter = 600; biYPelsPerMeter = 600;
         (* Unknown: the number of colors actually
            used by the image. Must be computed while writing the
            image. *)
-        biClrUsed = 0 ;
+        biClrUsed = 0;
         (* Number of important colors. If 0, all colors are important *)
         biClrImportant = 0 } in
 
@@ -869,14 +861,13 @@ let bmp_of_image img =
   | Index8 bitmap ->
     let colormap = bitmap.Index8.colormap.map
     and biW = bitmap.Index8.width
-    and biH = bitmap.Index8.height
-    and data = Bitmap.dump bitmap.Index8.data
-    in 
+    and biH = bitmap.Index8.height 
+    and data = Index8.dump bitmap in 
     let bfh = {
-      (* WORD *) bfType = 19778 (* BM *) ;
+      (* WORD *) bfType = 19778 (* BM *);
       (* DWORD *) bfSize = -1 (* Unknown to be updated *);
-      (* WORD *) bfReserved1 = 0 ;
-      (* WORD *) bfReserved2 = 0 ;
+      (* WORD *) bfReserved1 = 0;
+      (* WORD *) bfReserved2 = 0;
       (* DWORD *) bfOffBits = -1 (* Unknown to be updated *)
     } in
     let biBitCount,biClrUsed,biCompression,biClrImportant = 
@@ -889,17 +880,17 @@ let bmp_of_image img =
       | n when n <= 256 -> Color256, col_map_len, BI_RLE8, 0
       | n -> failwith "Too many colors for a bitmap with 8 bits per pixel" in
      let bih =
-       { biSize = -1 ; biWidth = biW ; biHeight = biH ;
-         biPlanes = 1 ; biBitCount = biBitCount;
-         biCompression = biCompression ; biSizeImage = -1 ;
-         biXPelsPerMeter = 600 ; biYPelsPerMeter = 600 ;
-         biClrUsed = biClrUsed ; biClrImportant = biClrImportant } in
+       { biSize = -1; biWidth = biW; biHeight = biH;
+         biPlanes = 1; biBitCount = biBitCount;
+         biCompression = biCompression; biSizeImage = -1;
+         biXPelsPerMeter = 600; biYPelsPerMeter = 600;
+         biClrUsed = biClrUsed; biClrImportant = biClrImportant; } in
 
      { bmpFileHeader = bfh;
        bmpInfoHeader = bih;
        bmpRgbQuad = colormap;
-       bmpBytes = data }
-  | _ -> raise Wrong_image_type
+       bmpBytes = data; }
+  | _ -> raise Wrong_image_type;;
 
 let save_bmp filename = function
  { bmpFileHeader = bmpFileHeader;
@@ -936,12 +927,14 @@ let save_bmp filename = function
  seek_out oc biSizeImage_index;
  output_dword oc biSizeImage;
 
- close_out oc
-;;
+ close_out oc;;
 
 let save filename opts img = save_bmp filename (bmp_of_image img);;
 
 add_methods Bmp
  { check_header = check_header; 
    load = Some load;
-   save = Some save };;
+   save = Some save;
+   load_sequence = None;
+   save_sequence = None;
+ };;
