@@ -60,9 +60,7 @@ let blend = function
         let t = (0xff - dst) * src + dst * (0xff - src) in
         let t = t + 0x80 in
         let t = t + t lsr 8 in
-        t lsr 8
-  | _ -> assert false
-;;
+        t lsr 8;;
 
 open Color
 
