@@ -6,19 +6,20 @@
 (*            Pierre Weis, projet Cristal, INRIA Rocquencourt          *)
 (*            Jun Furuse, projet Cristal, INRIA Rocquencourt           *)
 (*                                                                     *)
-(*  Copyright 1999,2000,2001,2002,2001,2002                            *)
+(*  Copyright 1999-2004,                                               *)
 (*  Institut National de Recherche en Informatique et en Automatique.  *)
 (*  Distributed only by permission.                                    *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* Image misc infos (copied to image.ml/mli *)
+(* $Id: info.mli,v 1.14 2004/09/07 07:31:02 weis Exp $ *)
+
+(* Image misc infos (copied to images.ml/mli *)
 
 (* File header informations *)
 
 type colormodel =
-   | Gray | RGB | Index | GrayA | RGBA
-;;
+   | Gray | RGB | Index | GrayA | RGBA;;
 
 (* Infos attached to bitmaps *)
 type info =
@@ -26,5 +27,4 @@ type info =
    | Info_BigEndian | Info_LittleEndian (* endianness of image file *)
    | Info_ColorModel of colormodel (* color model of image file *)
    | Info_Depth of int (* Image bit depth *)
-   | Info_Corrupted (* For corrupted PNG files *)
-;;
+   | Info_Corrupted (* For corrupted PNG files *);;
