@@ -4,9 +4,12 @@
 (*                                                                     *)
 (*            Jun Furuse, projet Cristal, INRIA Rocquencourt           *)
 (*                                                                     *)
-(*  Copyright 1999,2000,2001,2002,2001,2002                            *)
+(*  Copyright 1999-2004,                                               *)
 (*  Institut National de Recherche en Informatique et en Automatique.  *)
 (*  Distributed only by permission.                                    *)
 (*                                                                     *)
 (***********************************************************************)
-val encode : string -> int array
+
+(* $Id: jis_unicode.mli,v 1.4 2004/09/27 08:10:18 weis Exp $ *)
+
+val encode : string -> int array;;
