@@ -4,19 +4,19 @@
 (*                                                                     *)
 (*            Jun Furuse, projet Cristal, INRIA Rocquencourt           *)
 (*                                                                     *)
-(*  Copyright 1999,2000,2001,2002,2001,2002                            *)
+(*  Copyright 1999, 2004                                               *)
 (*  Institut National de Recherche en Informatique et en Automatique.  *)
 (*  Distributed only by permission.                                    *)
 (*                                                                     *)
 (***********************************************************************)
-open OImage
 
-let load name opts =
-  OImage.make (Ppm.load name opts)
-;;
+(* $Id: oPpm.ml,v 1.4 2004/09/07 07:31:07 weis Exp $ *)
+
+open OImages;;
+
+let load name opts = OImages.make (Ppm.load name opts);;
 
 let save name opts image =
   match image#image_class with
   | ClassRgb24 | ClassIndex8 -> Ppm.save name opts image#image
-  | _ -> raise Wrong_image_class
-;;
+  | _ -> raise Wrong_image_class;;
